PUBLIC_API_URL = "https://indices-backend.edhecinfra.com"
METRICS_URL = f"{PUBLIC_API_URL}/data-service/v2/queries/metrics"
INFRA_EQUITY_COMPARABLE_URL = f"{PUBLIC_API_URL}/compute-service/v2/pi-comparables/average/by/profiles"
INFRA_DEBT_COMPARABLE_URL = f"{PUBLIC_API_URL}/compute-service/v2/pi-comparables/average/by/profiles"
TERMSTRUCTURE_URL = f"{PUBLIC_API_URL}/data-service/v2/yields"
INDICES_CATALOGUE_URL = f"{PUBLIC_API_URL}/data-service/v2/lists/indices"
TAXONOMIES_URL = f"{PUBLIC_API_URL}/data-service/v2/lists/taxonomy/{{taxonomy}}"
COUNTRIES_URL = f"{PUBLIC_API_URL}/data-service/v2/yields/countries"
PRIVATE_EQUITY_COMPARABLE_URL = f"{PUBLIC_API_URL}/compute-service/v2/pe-comparables/average/by/profiles"
PRIVATE_EQUITY_COMPARABLE_BOUNDARIES_URL = f"{PUBLIC_API_URL}/compute-service/v2/pe-comparables/boundaries/by/profiles"
PRIVATE_EQUITY_REGION_TREE_URL = f"{PUBLIC_API_URL}/compute-service/v2/pe-comparables/region-tree"
PUBLIC_API_VERSION = "v2"
SOURCE = "sipaMetrics-python-sdk"
